unit uphoto;

interface

uses
  Types;

type

  TPhoto = class
  private
    FFocus: real;
    FY: real;
    FZ: real;
    FX: real;
    FPente: real;
    FAzimut: real;
    FTilt: real;
    FLargeur: integer;
    FHauteur: integer;
    procedure SetAzimut(const Value: real);
    procedure SetFocus(const Value: real);
    procedure SetPente(const Value: real);
    procedure SetTilt(const Value: real);
    procedure SetX(const Value: real);
    procedure SetY(const Value: real);
    procedure SetZ(const Value: real);
    procedure SetHauteur(const Value: integer);
    procedure SetLargeur(const Value: integer);
  public
    property X : real read FX write SetX;  // coordonne X (en km) de l'appareil
    property Y : real read FY write SetY;  // coordonne Y (en km) de l'appareil
    property Z : real read FZ write SetZ;  // coordonne Z (en km) de l'appareil
    property Azimut : real read FAzimut write SetAzimut; // azimuth de la prise de vue en radian (origine et sens d'une boussole)
    property Pente : real read FPente write SetPente; // pente de la prise de vue en radian
    property Tilt : real read FTilt write SetTilt; // inclinaison de l'appareil (non gre)
    property Focus : real read FFocus write SetFocus; // focale quivalente  un  appareil 24x36 mm
    property Largeur : integer read FLargeur write SetLargeur; // largeur en pixels de la photo
    property Hauteur : integer read FHauteur write SetHauteur; // hauteur en pixels de la photo
    constructor Create;
    destructor Free;
    function Terrain2Photo(TX, TY, TZ : real) : TPoint; // conversion des coordonnes d'un point sur le terrain en sa projection sur la photo
    procedure AddPtRef(TX, TY, TZ : real; PixelX, PixelY : integer); // ajout d'un point de repre
  end;

implementation

uses
  math;

{ TPhoto }

procedure TPhoto.AddPtRef(TX, TY, TZ: real; PixelX, PixelY: integer);
// TX, TY, TZ les coordonnes du point sur le terrain
// PixelX et PixelY, sa position sur la photo.
var
  VPhotoX, VPhotoY, VPhotoZ : real; // direction du point dans le repre de la photo
  VTerrainX, VTerrainY, VTerrainZ : real; // direction du point sur le terrain
  AzPhoto, AzTerrain : real;
  FocusPixel, Norme : real;
  a, b, c, Delta : real;
begin
  VTerrainX:=TX-X;
  VTerrainY:=TY-Y;
  VTerrainZ:=TZ-Z;
  Norme:=sqrt(sqr(VTerrainX)+sqr(VTerrainY)+sqr(VTerrainZ));
  VTerrainX:=VTerrainX/Norme;
  VTerrainY:=VTerrainY/Norme;
  VTerrainZ:=VTerrainZ/Norme;

  // conversion du focus de millimtre vers pixel
  FocusPixel:=sqrt(sqr(Largeur)+sqr(Hauteur)) * Focus / 43.2666; // 43.2666 mm est la diagonale du 24x36 mm.

  VPhotoX:=PixelX-Largeur/2;
  VPhotoY:=FocusPixel;
  VPhotoZ:=Hauteur/2-PixelY;
  Norme:=sqrt(sqr(VPhotoX)+sqr(VPhotoY)+sqr(VPhotoZ));
  VPhotoX:=VPhotoX/Norme;
  VPhotoY:=VPhotoY/Norme;
  VPhotoZ:=VPhotoZ/Norme;

  // quation  rsoudre VTerrainZ = VPhotoZ * cos(Pente) - VPhotoY * sin(Pente)
  // on passe par une quation du second degr en tan(Pente)
  a:=sqr(VPhotoY)-sqr(VTerrainZ);
  b:=-2*VPhotoY*VPhotoZ;
  c:=sqr(VPhotoZ)-sqr(VTerrainZ);
  Delta:=sqr(b)-4*a*c;
  if VTerrainZ>0 then
    Pente:=-arctan((-b-sqrt(Delta))/2/a)
  else
    Pente:=-arctan((-b+sqrt(Delta))/2/a);

  // calcul de l'azimut
  VPhotoY:=VPhotoY*cos(Pente) + VPhotoZ * sin(Pente); // on commence par tourner
  AzPhoto:=arctan2(VPhotoY,VPhotoX);
  AzTerrain:=arctan2(VTerrainY,VTerrainX);
  Azimut:=AzPhoto-AzTerrain;
end;

constructor TPhoto.Create;
begin
  inherited Create;
// on met des valeurs bidon par dfaut pour limiter les problmes d'erreur ultrieurs
  Focus := 35 ;
  Largeur := 800;
  Hauteur := 533;
end;

destructor TPhoto.Free;
begin
  inherited Free;
end;

procedure TPhoto.SetAzimut(const Value: real);
begin
  FAzimut := Value;
end;

procedure TPhoto.SetFocus(const Value: real);
begin
  FFocus := Value;
end;

procedure TPhoto.SetHauteur(const Value: integer);
begin
  FHauteur := Value;
end;

procedure TPhoto.SetLargeur(const Value: integer);
begin
  FLargeur := Value;
end;

procedure TPhoto.SetPente(const Value: real);
begin
  FPente := Value;
end;

procedure TPhoto.SetTilt(const Value: real);
begin
  FTilt := Value;
end;

procedure TPhoto.SetX(const Value: real);
begin
  FX := Value;
end;

procedure TPhoto.SetY(const Value: real);
begin
  FY := Value;
end;

procedure TPhoto.SetZ(const Value: real);
begin
  FZ := Value;
end;

function TPhoto.Terrain2Photo(TX, TY, TZ: real): TPoint;
var
  UX, UY, UZ : real;
  FocusPixel : real; // focus en pixel
begin
  // centrage sur l'appareil photo
  TX:=TX-X;
  TY:=TY-Y;
  TZ:=TZ-Z;
  // rotation de l'azimut
  UX:= cos(Azimut)*TX - sin(Azimut)*TY;
  UY:= sin(Azimut)*TX + cos(Azimut)*TY;
  UZ:=TZ;
  // rotation de la pente
  TX:=UX;
  TY:= cos(Pente)*UY + sin(Pente)*UZ;
  TZ:=-sin(Pente)*UY + cos(Pente)*UZ;
  // conversion du focus de millimtre vers pixel
  FocusPixel:=sqrt(sqr(Largeur)+sqr(Hauteur)) * Focus / 43.2666; // 43.2666 mm est la diagonale du 24x36 mm.
  // projection sur le plan photo
  result.X:=round(TX/TY*FocusPixel)+Largeur div 2;
  result.Y:=Hauteur div 2 - round(TZ/TY*FocusPixel);
end;

end.
